<?php

namespace Axima\Sms\Controller\Adminhtml\Settings;

use Axima\Sms\Block\Adminhtml\AximaSmsBlock;
use Axima\Sms\Utils\Database;
use Axima\Sms\Utils\RedirectException;
use Axima\SmsGate\Client as SmsClient;
use Axima\SmsGate\SmsGateException;
use Axima\SmsGate\Validators;
use DateTime;
use Exception;
use GuzzleHttp\Client;
use Magento\Backend\App\Action;
use Magento\Framework\Controller\Result\Redirect;
use Magento\Framework\Controller\ResultFactory;
use Magento\Framework\Exception\FileSystemException;
use Magento\Framework\Phrase;
use Magento\Framework\Stdlib\DateTime\TimezoneInterface;
use Magento\Sales\Api\Data\OrderInterface;
use Magento\Sales\Model\ResourceModel\Order\Status\CollectionFactory;

/**
 * @author Tomáš Blatný
 */
class Index extends Action
{

	const DOMAIN = 'axima_sms';
	const TABLE_MESSAGES = 'axima_sms_messages';
	const CONFIG_PATH = __DIR__ . '/../../../../../../../var/config_axima_sms.json';

	const STATUS_SENT = 0;
	const STATUS_DELIVERED = 1;
	const STATUS_ERROR = 2;

	/** @var Database */
	private $database;

	/** @var TimezoneInterface */
	private $timezone;

	public function execute()
	{
		$this->database = new Database($this->_objectManager->get('\Magento\Framework\App\ResourceConnection'));
		$this->timezone = $this->_objectManager->get('\Magento\Framework\Stdlib\DateTime\TimezoneInterface');

		try {
			$this->resolveRequest();
		} catch (RedirectException $e) {
			/** @var Redirect $redirect */
			$redirect = $this->resultFactory->create(ResultFactory::TYPE_REDIRECT);
			$redirect->setUrl($e->getUrl());
			return $redirect;
		}

		$this->_view->loadLayout();
		$this->_view->renderLayout();
		return NULL;
	}


	public function redirect($action = NULL, $other = NULL)
	{
		throw new RedirectException($this->link($action, $other));
	}


	public function link($action = NULL, $other = NULL)
	{
		$url = $this->getUrl('Sms/settings');
		if ($action === NULL && $other === NULL) {
			return $url;
		}
		return $url . '?' . http_build_query(['action' => $action]) . $other;
	}


	public function resolveRequest()
	{
		$action = isset($_GET['action']) ? $_GET['action'] : 'default';
		if (!in_array($action, array('default', 'logs', 'settings'), TRUE)) {
			$this->redirect('default');
		}
		$method = 'render' . ucfirst($action);
		$data = array();
		if (method_exists($this, $method)) {
			$data = (array) $this->$method();
		}
		$that = $this;

		AximaSmsBlock::setTemplateData(array(
			'page' => $action,
			'date' => function ($date, $format = 'j.n.Y G:i:s') use ($that) {
				try {
					$dateTimeZone = new \DateTimeZone($that->timezone->getDefaultTimezone());
					$date = new DateTime($date, $dateTimeZone);
					return $date->format($format);
				} catch (Exception $e) {
					return '-';
				}
			},
			'ready' => $this->isClientSet(),
			'domain' => self::DOMAIN,
			'link' => function ($action, $other = NULL) use ($that) {
				return $that->link($action, $other);
			},
			'shortCodes' => $this->getShortCodes(),
		) + $data);
	}


	private function renderDefault()
	{
		$error = NULL;
		if (isset($_GET['check']) && $this->isClientSet()) {
			$client = $this->getSmsClient();
			try {
				$stats = $client->getAccountStatus();
				$date = new DateTime;
				$this->updateOptions(array(
					'credit' => $stats['credit'],
					'credit_last_update' => $date->format('Y-m-d G:i:s'),
				));
			} catch (SmsGateException $e) {}
			$this->redirect();
		}
		if (isset($_POST['_send']) && $this->isClientSet()) {
			$number = $_POST['number'];
			$text = $_POST['text'];
			if (!Validators::validateNumber($number)) {
				$error = 'Invalid phone number';
			} elseif (!trim($text)) {
				$error = 'Please provide SMS text.';
			} else {
				$this->sendSms($number, $text);
				$this->redirect(NULL, '&sent=1');
			}
		}
		$lastWeek = new DateTime('-1 week');
		$lastWeek = $lastWeek->format('Y-m-d G:i:s');
		return array(
			'text' => isset($text) ? $text : $this->getQuery('text'),
			'number' => isset($number) ? $number : $this->getQuery('number'),
			'error' => $error,
			'sent' => $this->getQuery('sent'),
			'totalSent' => $this->database->selectOne('COUNT(*)', self::TABLE_MESSAGES, 'WHERE `status` IN (%d, %d)', array(self::STATUS_SENT, self::STATUS_DELIVERED)),
			'totalDelivered' => $this->database->selectOne('COUNT(*)', self::TABLE_MESSAGES, 'WHERE `status` = %d', array(self::STATUS_DELIVERED)),
			'lastWeekSent' => $this->database->selectOne('COUNT(*)', self::TABLE_MESSAGES, 'WHERE `status` IN (%d, %d) AND `date_sent` > %s', array(self::STATUS_SENT, self::STATUS_DELIVERED, $lastWeek)),
			'lastWeekDelivered' => $this->database->selectOne('COUNT(*)', self::TABLE_MESSAGES, 'WHERE `status` = %d AND `date_sent` > %s', array(self::STATUS_DELIVERED, $lastWeek)),
			'totalErrors' => $this->database->selectOne('COUNT(*)', self::TABLE_MESSAGES, 'WHERE `status` = %d', array(self::STATUS_ERROR)),
			'lastWeekErrors' => $this->database->selectOne('COUNT(*)', self::TABLE_MESSAGES, 'WHERE `status` = %d AND `date_sent` > %s', array(self::STATUS_ERROR, $lastWeek)),
			'lastCredit' => $this->getOption('credit'),
			'lastCreditUpdate' => $this->getOption('credit_last_update'),
		);
	}


	private function renderLogs()
	{
		$page = (int) $this->getQuery('list', 1);
		$count = (int) $this->database->selectOne('COUNT(*)', self::TABLE_MESSAGES);
		$perPage = 30;
		$pageCount = (int) ceil($count / $perPage);
		if ($page < 1) {
			$page = 1;
		}
		if ($page > $pageCount) {
			$page = $pageCount;
		}
		$pages = array(1, $pageCount, $page, $page - 1, $page - 2, $page + 1, $page + 2);
		if ($pageCount > 5) {
			for ($i = 1; $i <= $pageCount; $i += ($pageCount / 5)) {
				$pages[] = round($i);
			}
		}
		$pages = array_unique(array_filter($pages, function ($item) use ($pageCount) {
			return ($item >= 1) && ($item <= $pageCount);
		}));
		sort($pages);
		$results = $count ? $this->database->select(self::TABLE_MESSAGES, 'ORDER BY `date_sent` DESC LIMIT ' . $perPage . ' OFFSET ' . ($page - 1) * $perPage) : array();
		$lastCheck = $this->getOption('lastCheck');

		if (isset($_GET['check']) && $this->isClientSet()) {
			try {
				$this->checkForNewSms();
			} catch (SmsGateException $e) {}
			$this->redirect('logs', '&list=' . $page);
		}

		return array(
			'messages' => $results,
			'pages' => $pages,
			'list' => $page,
			'maxPage' => $pageCount,
			'lastCheck' => $lastCheck,
		);
	}


	private function renderSettings()
	{
		$error = '';
		if (isset($_POST['_submit'])) {
			$password = trim($this->getPost('password'));
			$enabled = array();
			$enabledData = $this->getPost('enabled', array());
			foreach ($this->getStatuses() as $status => $name) {
				$enabled[$status] = isset($enabledData[$status]) && $enabledData[$status] === 'on';
			}
			$update = array(
				'name' => trim($this->getPost('name')),
				'texts' => $this->getPost('texts'),
				'enabled' => $enabled,
			);
			if ($password) {
				$update['password'] = $password;
			}
			$this->updateOptions($update);
		}
		if ($this->getQuery('logout')) {
			$this->updateOptions(array(
				'name' => NULL,
				'password' => NULL,
			));
			$this->redirect();
		}

		return array(
			'error' => $error,
			'settings' => $this->getOptions(),
			'statuses' => $this->getStatuses(),
		);
	}



	// -----------------------------


	private function checkForNewSms()
	{
		$client = $this->getSmsClient();
		$reports = $client->getDeliveryReports();
		foreach ($reports as $id => $timestamp) {
			$date = date_create_from_format('YmdGis', $timestamp);
			$this->database->update(self::TABLE_MESSAGES, array('status' => self::STATUS_DELIVERED, 'date_received' => $date->format('Y-m-d G:i:s')), '`sms_id` = %s', array($id));
			$client->confirmDeliveryReport($id);
		}
		$date = new DateTime;
		$this->updateOptions(array('lastCheck' => $date->format('Y-m-d G:i:s')));
	}


	private function sendSms($number, $text)
	{
		$client = $this->getSmsClient();
		$date = new DateTime;
		$date = $date->format('Y-m-d G:i:s');
		try {
			$result = $client->sendSms($text, $number, TRUE);
			$this->database->insert(self::TABLE_MESSAGES, array(
				'sms_id' => $result['id'],
				'text' => $text,
				'number' => $number,
				'status' => self::STATUS_SENT,
				'date_sent' => $date,
			));
			$this->updateOptions(array(
				'credit' => $result['credit'],
				'credit_last_update' => $date,
			));
		} catch (Exception $exception) {
			$note = '';
			if ($exception instanceof SmsGateException) {
				$note = $exception->getMessage();
			}
			$this->database->insert(self::TABLE_MESSAGES, array(
				'sms_id' => NULL,
				'text' => $text,
				'number' => $number,
				'status' => self::STATUS_ERROR,
				'note' => $note,
				'date_sent' => $date,
			));
		}

	}


	private function updateOptions(array $options)
	{
		$current = $this->getOptions();
		foreach ($options as $key => $option) {
			$current[$key] = $option;
		}
		if (!@file_put_contents(self::CONFIG_PATH, json_encode($current))) {
			throw new FileSystemException(new Phrase('Cannot write to config file, please check if Magento has write access to var directory.'));
		}
	}


	/**
	 * @return array
	 */
	private function getOptions()
	{
		$config = @file_get_contents(self::CONFIG_PATH);
		if (!$config) {
			$config = array();
		} else {
			$config = json_decode($config, JSON_OBJECT_AS_ARRAY);
		}
		return $config;
	}


	/**
	 * @param string $name
	 * @param mixed $default
	 * @return mixed
	 */
	private function getOption($name, $default = NULL)
	{
		$options = $this->getOptions();
		return isset($options[$name]) ? $options[$name] : $default;
	}


	private function getStatuses()
	{
		/** @var CollectionFactory $collectionFactory */
		$collectionFactory = $this->_objectManager->get('\Magento\Sales\Model\ResourceModel\Order\Status\CollectionFactory');
		$items = $collectionFactory->create()->toOptionArray();
		$result = array();
		foreach ($items as $item) {
			$result[$item['value']] = $item['label'];
		}
		return $result;
	}


	/**
	 * @param string $name
	 * @param mixed $default
	 * @return mixed
	 */
	private function getPost($name, $default = NULL)
	{
		return isset($_POST[$name]) && $_POST[$name] ? $_POST[$name] : $default;
	}


	private function getQuery($name, $default = NULL)
	{
		return isset($_GET[$name]) && $_GET[$name] ? $_GET[$name] : $default;
	}


	private function isClientSet()
	{
		return $this->getOption('name') !== NULL && $this->getOption('password') !== NULL;
	}


	private function getSmsClient()
	{
		return new SmsClient(new Client, $this->getOption('name'), $this->getOption('password'));
	}


	private function getShortCodes(OrderInterface $order = NULL)
	{
		if (!$order) {
			return array( // dummy data
				'name' => 'John Doe',
				'billingAddress' => 'Narrow Street 145, London, B45 0HY',
				'price' => '1,785.43 EUR',
				'paymentMethod' => 'Bank transfer',
			);
		}
		return array(
			'name' => $order->getCustomerFirstname() . ' ' . $order->getCustomerLastname(),
			'billingAddress' => implode(' ', $order->getBillingAddress()->getStreet()) . ' ' . $order->getBillingAddress()->getCity() . ', ' . $order->getBillingAddress()->getPostcode(),
			'price' => $order->getTotalInvoiced() . $order->getOrderCurrencyCode(),
			'paymentMethod' => $order->getPayment()->getMethod(),
		);
	}


}
